<?php
/**
 * This file is part of the official Amazon Checkout extension for Magento 1.x
 *
 * (c) 2020 creativestyle GmbH. All Rights reserved
 *
 * Distribution of the derivatives reusing, transforming or being built upon
 * this software, is not allowed without explicit written permission granted
 * by creativestyle GmbH
 *
 * @category   Creativestyle
 * @package    Creativestyle_AmazonCheckout
 * @copyright  2020 creativestyle GmbH
 * @author     Marek Stefan <ticket@creativestyle.de>
 */

class Creativestyle_AmazonCheckout_Block_Template extends Mage_Core_Block_Template
{
    /**
     * @var string|null
     */
    protected $_checkoutSessionId = null;

    /**
     * Instance of the current quote
     *
     * @var Mage_Sales_Model_Quote
     */
    protected $_quote = null;

    /**
     * Return instance of Amazon Pay config object
     *
     * @return Creativestyle_AmazonCheckout_Model_Config
     */
    protected function _getConfig()
    {
        /** @var Creativestyle_AmazonCheckout_Model_Config $config */
        $config = Mage::getSingleton('amazonpay/config');
        return $config;
    }

    /**
     * Return instance of Amazon Pay API adapter
     *
     * @return Creativestyle_AmazonCheckout_Model_Api
     */
    protected function _getApi()
    {
        /** @var Creativestyle_AmazonCheckout_Model_Api $api */
        $api = Mage::getSingleton('amazonpay/api');
        return $api;
    }

    /**
     * Return instance of Amazon Pay URL model
     *
     * @return Creativestyle_AmazonCheckout_Model_Url
     */
    protected function _getUrlModel()
    {
        /** @var Creativestyle_AmazonCheckout_Model_Url $urlModel */
        $urlModel = Mage::getSingleton('amazonpay/url');
        return $urlModel;
    }

    /**
     * Return instance of Amazon Pay helper
     *
     * @return Creativestyle_AmazonCheckout_Helper_Data
     */
    protected function _getAmazonHelper()
    {
        /** @var Creativestyle_AmazonCheckout_Helper_Data $helper */
        $helper = $this->helper('amazonpay');
        return $helper;
    }

    /**
     * Return instance of Magento core helper
     *
     * @return Mage_Core_Helper_Data
     */
    protected function _getCoreHelper()
    {
        /** @var Mage_Core_Helper_Data $helper */
        $helper = $this->helper('core');
        return $helper;
    }

    /**
     * Returns instance of the checkout session
     *
     * @return Mage_Checkout_Model_Session
     */
    protected function _getCheckoutSession()
    {
        /** @var Mage_Checkout_Model_Session $session */
        $session = Mage::getSingleton('checkout/session');
        return $session;
    }

    /**
     * Returns instance of the current quote
     *
     * @return Mage_Sales_Model_Quote
     */
    protected function _getQuote()
    {
        if (null === $this->_quote) {
            $this->_quote = $this->_getCheckoutSession()->getQuote();
        }

        return $this->_quote;
    }

    /**
     * Encode provided $valueToEncode into the JSON format
     *
     * @param mixed $valueToEncode
     * @return string
     */
    protected function _jsonEncode($valueToEncode)
    {
        return $this->_getCoreHelper()->jsonEncode($valueToEncode);
    }

    /**
     * Render block HTML
     *
     * @return string
     */
    protected function _toHtml()
    {
        if (!$this->isActive()) {
            return '';
        }
        return parent::_toHtml();
    }

    /**
     * Checks whether requester IP is allowed to display Amazon widgets
     *
     * @return bool
     */
    protected function _isCurrentIpAllowed()
    {
        $allowedIps = $this->_getConfig()->getAllowedIps();

        if (null !== $allowedIps) {
            $currentIp = Mage::helper('core/http')->getRemoteAddr();
            if (Mage::app()->getRequest()->getServer('HTTP_X_FORWARDED_FOR')) {
                $currentIp = Mage::app()->getRequest()->getServer('HTTP_X_FORWARDED_FOR');
            }

            return in_array($currentIp, $allowedIps);
        }

        return true;
    }

    /**
     * Return flag whether the block shall be displayed or not
     *
     * @return string
     * @api
     */
    public function isActive()
    {
        return ($this->_getConfig()->isPayActive() || $this->_getConfig()->isLoginActive())
            && $this->_isCurrentIpAllowed()
            && $this->_getConfig()->isPrivateKeyValid();
    }

    /**
     * Return current version of the extension
     *
     * @return string
     * @api
     */
    public function getVersion()
    {
        return $this->_getConfig()->getVersion();
    }

    /**
     * Return the configured Amazon Merchant ID
     *
     * @return string
     * @api
     */
    public function getMerchantId()
    {
        return $this->_getConfig()->getMerchantId();
    }

    /**
     * Return the configured Amazon client ID
     *
     * @return string
     * @api
     */
    public function getClientId()
    {
        return $this->_getConfig()->getClientId();
    }

    /**
     * Return the configured Amazon public key ID
     *
     * @return string
     * @api
     */
    public function getPublicKeyId()
    {
        return $this->_getConfig()->getPublicKeyId();
    }

    /**
     * Return flag whether Amazon Pay runs in sandbox mode
     *
     * @return bool
     * @api
     */
    public function isSandboxActive()
    {
        return $this->_getAmazonHelper()->isSandboxActive();
    }

    /**
     * Return the language for Amazon Pay widgets
     *
     * @return string
     * @api
     */
    public function getLanguage()
    {
        $language = $this->_getConfig()->getLanguage();
        if (!$language) {
            /** @var Creativestyle_AmazonCheckout_Model_System_Config_Source_Language $languageSource */
            $languageSource = Mage::getSingleton('amazonpay/system_config_source_language');
            $language = $languageSource->getLanguageForLocale(Mage::app()->getLocale()->getLocaleCode());
        }

        return $language;
    }

    /**
     * Return configured color for Amazon Pay button
     *
     * @return string
     * @api
     */
    public function getPayButtonColor()
    {
        return $this->_getConfig()->getPayButtonColor();
    }

    /**
     * Return configured color for Loginw with Amazon button
     *
     * @return string
     * @api
     */
    public function getLoginButtonColor()
    {
        return $this->_getConfig()->getLoginButtonColor();
    }

    /**
     * Return presentment currency code for the checkout
     *
     * @return string|null
     * @api
     */
    public function getPresentmentCurrencyCode()
    {
        try {
            return Mage::app()->getStore()->getBaseCurrencyCode();
        } catch (Mage_Core_Model_Store_Exception $e) {
            return null;
        }
    }

    /**
     * Return merchant's ledger currency code
     *
     * @return string|null
     * @api
     */
    public function getLedgerCurrencyCode()
    {
        return $this->_getConfig()->getLedgerCurrency();
    }

    /**
     * @return bool
     * @api
     */
    public function isOrderVirtual()
    {
        return $this->_getQuote() ? $this->_getQuote()->isVirtual() : false;
    }

    /**
     * Return URL for add-to-cart XHR endpoint
     *
     * @return string
     * @api
     */
    public function getAddToCartUrl()
    {
        return $this->_getUrlModel()->getAddToCartUrl();
    }

    /**
     * Return store URL for creating checkout session
     *
     * @return string
     * @api
     */
    public function getCreateCheckoutSessionUrl()
    {
        return $this->_getUrlModel()->getCreateCheckoutSessionUrl();
    }

    /**
     * Return URL for sign-in callback page
     *
     * @return string
     * @api
     */
    public function getSignInReturnUrl()
    {
        return $this->_getUrlModel()->getSignInReturnUrl();
    }

    /**
     * @return string|null
     * @api
     */
    public function getCheckoutSessionId()
    {
        return $this->_checkoutSessionId;
    }

    /**
     * @param string $checkoutSessionId
     * @return $this
     * @api
     */
    public function setCheckoutSessionId($checkoutSessionId)
    {
        $this->_checkoutSessionId = $checkoutSessionId;
        return $this;
    }

    /**
     * Return store URL for saving order
     *
     * @return string
     * @api
     */
    public function getSaveShippingMethodUrl()
    {
        return $this->_getUrlModel()->getSaveShippingMethodUrl();
    }

    /**
     * Return URL for handling discount coupons
     *
     * @return string
     * @api
     */
    public function getCouponPostUrl()
    {
        return $this->_getUrlModel()->getCouponPostUrl();
    }

    /**
     * Return store URL for saving order
     *
     * @return string|null
     * @api
     */
    public function getSaveOrderDataUrl()
    {
        return $this->_getUrlModel()->getSaveOrderDataUrl();
    }

    /**
     * Return store URL for placing order
     *
     * @return string|null
     * @api
     */
    public function getPlaceOrderUrl()
    {
        return $this->getCheckoutSessionId()
            ? $this->_getUrlModel()->getPlaceOrderUrl($this->getCheckoutSessionId())
            : null;
    }
}
