<?php
/**
 * This file is part of the official Amazon Checkout extension for Magento 1.x
 *
 * (c) 2020 creativestyle GmbH. All Rights reserved
 *
 * Distribution of the derivatives reusing, transforming or being built upon
 * this software, is not allowed without explicit written permission granted
 * by creativestyle GmbH
 *
 * @category   Creativestyle
 * @package    Creativestyle_AmazonCheckout
 * @copyright  2020 creativestyle GmbH
 * @author     Marek Stefan <ticket@creativestyle.de>
 */

class Creativestyle_AmazonCheckout_Model_Config
{
    const XML_PATH_MERCHANT_ID          = 'amazonpay/account/merchant_id';
    const XML_PATH_CLIENT_ID            = 'amazonpay/account/client_id';
    const XML_PATH_PRIVATE_KEY          = 'amazonpay/account/private_key';
    const XML_PATH_PUBLIC_KEY_ID        = 'amazonpay/account/public_key_id';
    const XML_PATH_ACCOUNT_REGION       = 'amazonpay/account/region';
    const XML_PATH_USE_LEGACY           = 'amazonpay/account/use_legacy';

    const XML_PATH_ACTIVE               = 'amazonpay/general/active';
    const XML_PATH_ENVIRONMENT          = 'amazonpay/general/environment';
    const XML_PATH_LANGUAGE             = 'amazonpay/general/language';
    const XML_PATH_PAYMENT_ACTION       = 'amazonpay/general/payment_action';
    const XML_PATH_LOGIN_ACTIVE         = 'amazonpay/general/login_active';
    const XML_PATH_CHECKOUT_ACTIVE      = 'amazonpay/general/checkout_active';
    const XML_PATH_PDP_ACTIVE           = 'amazonpay/general/product_page_active';

    const XML_PATH_PAY_BUTTON_COLOR     = 'amazonpay/design/pay_button_color';
    const XML_PATH_LOGIN_BUTTON_COLOR   = 'amazonpay/design/login_button_color';

    const XML_PATH_STATUS_AUTHORIZED    = 'amazonpay/order_status/authorized';

    const XML_PATH_RESTRICT_PACKSTATION = 'amazonpay/delivery/restrict_packstation';
    const XML_PATH_RESTRICT_PO_BOX      = 'amazonpay/delivery/restrict_po_box';

    const XML_PATH_STORE_NAME           = 'amazonpay/store/name';
    const XML_PATH_SOFT_DESCRIPTOR      = 'amazonpay/store/soft_descriptor';

    const XML_PATH_ALEXA_ACTIVE         = 'amazonpay/alexa/active';
    const XML_PATH_ALEXA_CARRIER_CODES  = 'amazonpay/alexa/carrier_codes';

    const XML_PATH_LOG_ACTIVE           = 'amazonpay/developer/log_active';
    const XML_PATH_ALLOWED_IPS          = 'amazonpay/developer/allowed_ips';

    const XML_PATH_ALLOWED_COUNTRIES    = 'general/country/allow';

    const XML_GLOBAL_CONFIG_NODES_PATH  = 'global/creativestyle/amazonpayments';

    /**
     * Config nodes array
     *
     * @var array|null
     */
    protected $_globalConfigNodes = null;

    /**
     * @return Mage_Core_Model_Encryption
     */
    protected function _getEncryptionModel()
    {
        /** @var Mage_Core_Model_Encryption $encryption */
        $encryption = Mage::getSingleton('core/encryption');
        return $encryption;
    }

    /**
     * Return instance of Amazon Pay helper
     *
     * @return Creativestyle_AmazonCheckout_Helper_Data
     */
    protected function _getAmazonHelper()
    {
        /** @var Creativestyle_AmazonCheckout_Helper_Data $helper */
        $helper = Mage::helper('amazonpay');
        return $helper;
    }

    /**
     * Return config option value in global/creativestyle/amazonpayments node
     *
     * @param string|null $key
     * @return string|array
     */
    public function _getGlobalConfigNode($key = null)
    {
        if (null === $this->_globalConfigNodes) {
            $config = Mage::getConfig()->getNode(self::XML_GLOBAL_CONFIG_NODES_PATH);
            $this->_globalConfigNodes = $config ? $config->asArray() : array();
        }

        if (null !== $key) {
            if (array_key_exists($key, $this->_globalConfigNodes)) {
                return $this->_globalConfigNodes[$key];
            }

            return null;
        }

        return $this->_globalConfigNodes;
    }

    /**
     * Return current version of the extension
     *
     * @return string
     * @api
     */
    public function getVersion()
    {
        return (string)Mage::getConfig()->getNode('modules/Creativestyle_AmazonCheckout/version');
    }

    /**
     * Return the configured Amazon Merchant ID
     *
     * @api
     *
     * @param mixed|null $store
     * @return string
     */
    public function getMerchantId($store = null)
    {
        return trim(strtoupper(Mage::getStoreConfig(self::XML_PATH_MERCHANT_ID, $store)));
    }

    /**
     * Return the configured Amazon Client ID
     *
     * @api
     *
     * @param mixed|null $store
     * @return string
     */
    public function getClientId($store = null)
    {
        return trim(Mage::getStoreConfig(self::XML_PATH_CLIENT_ID, $store));
    }

    /**
     * @param mixed|null $store
     * @return string|null
     * @api
     */
    public function getPrivateKey($store = null)
    {
        $encryptedKey = Mage::getStoreConfig(self::XML_PATH_PRIVATE_KEY, $store);
        if ($encryptedKey) {
            return $this->_getEncryptionModel()->decrypt($encryptedKey);
        }

        return null;
    }

    /**
     * @param mixed|null $store
     * @return string|null
     * @api
     */
    public function isPrivateKeyValid($store = null)
    {
        return $this->_getAmazonHelper()->isPrivateKeyValid($this->getPrivateKey($store));
    }

    /**
     * @param mixed|null $store
     * @return string|null
     */
    public function getPublicKeyId($store = null)
    {
        return Mage::getStoreConfig(self::XML_PATH_PUBLIC_KEY_ID, $store);
    }

    /**
     * @return string
     */
    public function getPlatformId()
    {
        return 'AIOVPYYF70KB5';
    }

    /**
     * Return merchant's payment region
     *
     * @param mixed|null $store
     * @return string|null
     * @api
     */
    public function getPaymentRegion($store = null)
    {
        return Mage::getStoreConfig(self::XML_PATH_ACCOUNT_REGION, $store);
    }

    /**
     * Returns merchant's region
     *
     * @param mixed|null $store
     * @return string|null
     * @api
     */
    public function getRegion($store = null)
    {
        switch ($this->getPaymentRegion($store)) {
            case 'EUR':
            case 'EUR_DE':
            case 'EUR_FR':
            case 'EUR_IT':
            case 'EUR_ES':
                return 'de';
            case 'GBP':
                return 'uk';
            case 'USD':
                return 'us';
            case 'JPY':
                return 'jp';
            default:
                return null;
        }
    }

    /**
     * Returns merchant's ledger currency
     *
     * @param mixed|null $store
     * @return string|null
     * @api
     */
    public function getLedgerCurrency($store = null)
    {
        $paymentRegion = $this->getPaymentRegion($store);
        switch ($paymentRegion) {
            case 'EUR_DE':
            case 'EUR_FR':
            case 'EUR_IT':
            case 'EUR_ES':
                return 'EUR';
            default:
                return $paymentRegion;
        }
    }

    /**
     * Check whether Amazon Pay is enabled
     *
     * @param mixed|null $store
     * @return bool
     * @api
     */
    public function isPayActive($store = null)
    {
        return Mage::getStoreConfigFlag(self::XML_PATH_ACTIVE, $store);
    }

    /**
     * @return bool
     * @api
     */
    public function isAdditionalPaymentButtonActive($store = null)
    {
        return Mage::getStoreConfigFlag(self::XML_PATH_CHECKOUT_ACTIVE, $store);
    }

    /**
     * @param mixed|null $store
     * @return bool
     * @api
     */
    public function isPayActiveOnProductDetailsPage($store = null)
    {
        return Mage::getStoreConfigFlag(self::XML_PATH_PDP_ACTIVE, $store);
    }

    /**
     * Check whether Login with Amazon is enabled
     *
     * @param mixed|null $store
     * @return bool
     * @api
     */
    public function isLoginActive($store = null)
    {
        return Mage::getStoreConfigFlag(self::XML_PATH_LOGIN_ACTIVE, $store);
    }

    /**
     * Return configured run-time mode (sandbox / live)
     *
     * @param mixed|null $store
     * @return string
     * @api
     */
    public function getEnvironment($store = null)
    {
        return Mage::getStoreConfig(self::XML_PATH_ENVIRONMENT, $store);
    }

    /**
     * Return display language for Amazon Pay widgets
     *
     * @param mixed|null $store
     * @return string
     * @api
     */
    public function getLanguage($store = null)
    {
        return Mage::getStoreConfig(self::XML_PATH_LANGUAGE, $store);
    }

    /**
     * Return configured payment action
     *
     * @param mixed|null $store
     * @return string
     * @api
     */
    public function getPaymentAction($store = null)
    {
        return Mage::getStoreConfig(self::XML_PATH_PAYMENT_ACTION, $store);
    }

    /**
     * Check whether order amount shall be captured immediately after the authorization
     *
     * @param mixed|null $store
     * @return bool
     * @api
     */
    public function shallCaptureImmediately($store = null)
    {
        return $this->getPaymentAction($store) == Creativestyle_AmazonCheckout_Model_Payment::ACTION_AUTHORIZE_CAPTURE;
    }

    /**
     * Return authorization request mode (sync, async)
     *
     * @TODO
     *
     * @param mixed|null $store
     * @return string
     * @api
     */
    public function getAuthorizationMode($store = null)
    {
        return null;
    }

    /**
     * Check if authorization may be requested asynchronously
     *
     * @TODO: Extract literals to constants
     *
     * @param mixed|null $store
     * @return string
     * @api
     */
    public function isAsyncAuthAllowed($store = null)
    {
        return in_array($this->getAuthorizationMode($store), array('auto', 'asynchronous'));
    }

    /**
     * Return configured store name
     *
     * @param mixed|null $store
     * @return string
     * @throws Mage_Core_Model_Store_Exception
     * @api
     */
    public function getStoreName($store = null)
    {
        $storeName = Mage::getStoreConfig(self::XML_PATH_STORE_NAME, $store);
        $storeName = $storeName ? $storeName : Mage::app()->getStore($store)->getFrontendName();
        return $storeName;
    }

    /**
     * Return configured soft descriptor
     *
     * @param mixed|null $store
     * @return string
     * @throws Mage_Core_Model_Store_Exception
     * @api
     */
    public function getSoftDescriptor($store = null)
    {
        $softDescriptor = Mage::getStoreConfig(self::XML_PATH_SOFT_DESCRIPTOR, $store);
        $softDescriptor = $softDescriptor ? $softDescriptor : $this->getStoreName($store);
        return substr($softDescriptor, 0, 16);
    }

    /**
     * Return configured color of Amazon Pay button
     *
     * @param mixed|null $store
     * @return string
     * @api
     */
    public function getPayButtonColor($store = null)
    {
        return Mage::getStoreConfig(self::XML_PATH_PAY_BUTTON_COLOR, $store);
    }

    /**
     * Return configured color of Login with Amazon button
     *
     * @param mixed|null $store
     * @return string
     * @api
     */
    public function getLoginButtonColor($store = null)
    {
        return Mage::getStoreConfig(self::XML_PATH_LOGIN_BUTTON_COLOR, $store);
    }

    /**
     * Return allowed shipping countries
     *
     * @param mixed|null $store
     * @return array
     * @api
     */
    public function getAllowedCountries($store = null)
    {
        $allowedCountries = Mage::getStoreConfig(self::XML_PATH_ALLOWED_COUNTRIES, $store);
        return explode(',', $allowedCountries);
    }

    /**
     * Return status for the order with authorized payment
     *
     * @param mixed|null $store
     * @return string
     */
    public function getAuthorizedOrderStatus($store = null)
    {
        return Mage::getStoreConfig(self::XML_PATH_STATUS_AUTHORIZED, $store);
    }

    /**
     * @param mixed|null $store
     * @return bool
     * @api
     */
    public function isPoBoxRestricted($store = null)
    {
        return Mage::getStoreConfigFlag(self::XML_PATH_RESTRICT_PO_BOX, $store);
    }

    /**
     * @param mixed|null $store
     * @return bool
     * @api
     */
    public function isPackstationRestricted($store = null)
    {
        return Mage::getStoreConfigFlag(self::XML_PATH_RESTRICT_PACKSTATION, $store);
    }

    /**
     * @return bool
     * @api
     */
    public function isJsVersioningDisabled()
    {
        return (bool) $this->_getGlobalConfigNode('disable_js_versioning');
    }

    /**
     * @return array|null
     * @api
     */
    public function getCheckoutCustomFields()
    {
        return $this->_getGlobalConfigNode('custom_fields');
    }

    /**
     * @return bool
     * @api
     */
    public function isLegacyExtensionInstalled()
    {
        $legacyActive = strtolower((string)Mage::getConfig()->getNode('modules/Creativestyle_AmazonPayments/active'));

        if (!empty($legacyActive) && 'false' !== $legacyActive && 'off' !== $legacyActive) {
            return true;
        }

        return false;
    }

    /**
     * @return bool
     * @api
     */
    public function useLegacyExtension()
    {
        return $this->isLegacyExtensionInstalled() && Mage::getStoreConfigFlag(self::XML_PATH_USE_LEGACY);
    }

    /**
     * Checks whether Alexa Delivery Notification is enabled
     *
     * @param mixed|null $store
     * @return bool
     * @api
     */
    public function isDeliveryNotificationActive($store = null)
    {
        return Mage::getStoreConfigFlag(self::XML_PATH_ALEXA_ACTIVE, $store);
    }

    /**
     * @param mixed|null $store
     * @return array
     * @api
     */
    public function getDeliveryNotificationCarriers($store = null)
    {
        $carriersMap = array();
        $carrierCodes = Mage::getStoreConfig(self::XML_PATH_ALEXA_CARRIER_CODES, $store);
        if ($carrierCodes) {
            $carrierCodes = call_user_func('unserialize', $carrierCodes);
            foreach ($carrierCodes as $carrier) {
                if (isset($carrier['magento']) && isset($carrier['amazon'])) {
                    $carriersMap[$carrier['magento']] = $carrier['amazon'];
                }
            }
        }

        return $carriersMap;
    }

    /**
     * Checks whether logger is enabled
     *
     * @param mixed|null $store
     * @return bool
     */
    public function isLoggerActive($store = null)
    {
        return Mage::getStoreConfigFlag(self::XML_PATH_LOG_ACTIVE, $store);
    }

    /**
     * Return list of allowed IPs
     *
     * @param mixed|null $store
     * @return array|null
     * @api
     */
    public function getAllowedIps($store = null)
    {
        $allowedIps = trim(Mage::getStoreConfig(self::XML_PATH_ALLOWED_IPS, $store), ' ,');
        if ($allowedIps) {
            $allowedIps = explode(',', str_replace(' ', '', $allowedIps));
            return $allowedIps;
        }

        return null;
    }
}
