<?php
/**
 * This file is part of the official Amazon Checkout extension for Magento 1.x
 *
 * (c) 2020 creativestyle GmbH. All Rights reserved
 *
 * Distribution of the derivatives reusing, transforming or being built upon
 * this software, is not allowed without explicit written permission granted
 * by creativestyle GmbH
 *
 * @category   Creativestyle
 * @package    Creativestyle_AmazonCheckout
 * @copyright  2020 creativestyle GmbH
 * @author     Marek Stefan <ticket@creativestyle.de>
 */

class Creativestyle_AmazonCheckout_Model_Payment extends Mage_Payment_Model_Method_Abstract
{
    const ACTION_AUTHORIZE                  = 'authorize';
    const ACTION_AUTHORIZE_CAPTURE          = 'authorize_capture';

    protected $_code                        = 'amazon_checkout';
    protected $_formBlockType               = 'amazonpay/payment_form';
    protected $_infoBlockType               = 'amazonpay/payment_info';

    protected $_isGateway                   = false;
    protected $_canOrder                    = true;
    protected $_canAuthorize                = true;
    protected $_canCapture                  = true;
    protected $_canCapturePartial           = true;
    protected $_canCaptureOnce              = true;
    protected $_canRefund                   = true;
    protected $_canRefundInvoicePartial     = true;
    protected $_canVoid                     = false;
    protected $_canUseInternal              = false;
    protected $_canUseCheckout              = true;
    protected $_canUseForMultishipping      = false;
    protected $_isInitializeNeeded          = true;
    protected $_canFetchTransactionInfo     = true;
    protected $_canReviewPayment            = true;
    protected $_canCreateBillingAgreement   = false;
    protected $_canManageRecurringProfiles  = false;

    /**
     * @return Creativestyle_AmazonCheckout_Model_Api
     */
    protected function _getApi()
    {
        /** @var Creativestyle_AmazonCheckout_Model_Api $api */
        $api = Mage::getSingleton('amazonpay/api');
        return $api;
    }

    /**
     * @return Creativestyle_AmazonCheckout_Model_Config
     */
    protected function _getConfig()
    {
        /** @var Creativestyle_AmazonCheckout_Model_Config $config */
        $config = Mage::getSingleton('amazonpay/config');
        return $config;
    }

    /**
     * @inheritdoc
     */
    public function getConfigData($field, $storeId = null)
    {
        switch ($field) {
            case 'payment_action':
                return $this->getConfigPaymentAction($storeId);
            default:
                return parent::getConfigData($field, $storeId);
        }
    }

    public function isAvailable($quote = null)
    {
        $checkResult = new StdClass;
        $isActive = $this->_getConfig()->isPayActive($quote ? $quote->getStoreId() : null);

        $checkResult->isDeniedInConfig = !$isActive;

        if ($quote && !$quote->validateMinimumAmount()) {
            $isActive = false;
        }

        if ($quote && Mage::app()->getStore()->roundPrice($quote->getGrandTotal()) == 0) {
            $isActive = false;
        }

        $checkResult->isAvailable = $isActive;

        Mage::dispatchEvent(
            'payment_method_is_active',
            array(
                'result' => $checkResult,
                'method_instance' => $this,
                'quote' => $quote,
            )
        );
        return $checkResult->isAvailable;
    }

    public function getConfigPaymentAction($store = null)
    {
        return $this->_getConfig()->getPaymentAction($store);
    }

    public function initialize($paymentAction, $stateObject)
    {
        $payment = $this->getInfoInstance();
        /** @var Mage_Sales_Model_Order $order */
        $order = $payment->getOrder();

        $checkoutSession = $this->_getApi()->completeCheckoutSession(
            $payment->getTransactionId(),
            $order->getBaseGrandTotal(),
            $order->getBaseCurrencyCode()
        );

        if ($checkoutSession->getState() == 'Canceled') {
            throw new Creativestyle_AmazonCheckout_Exception_Runtime_Payment_InvalidStatus();
        }

        $orderStatus = Mage::getModel('sales/order_status')
            ->setStore($order->getStore())
            ->loadDefaultByState(Mage_Sales_Model_Order::STATE_NEW)
            ->getStatus();

        $order->setState(
            Mage_Sales_Model_Order::STATE_NEW,
            $orderStatus,
            sprintf(
                'A checkout session has been processed by Amazon Payments (%s). The new status is %s.',
                $checkoutSession->getCheckoutSessionId(),
                $checkoutSession->getState()
            )
        );

        $payment->setAdditionalInformation(
            'amazon_order_reference_id',
            $checkoutSession->getChargePermissionId()
        );
        $order->setExtOrderId($checkoutSession->getChargePermissionId());
        $chargePermission = $this->_getApi()->getChargePermission($checkoutSession->getChargePermissionId());

        /** @var Mage_Sales_Model_Order_Payment_Transaction $auth */
        $auth = $payment->setIsTransactionClosed(false)
            ->setTransactionId($chargePermission['chargePermissionId'])
            ->addTransaction(Mage_Sales_Model_Order_Payment_Transaction::TYPE_AUTH);

        $payment->setAmountAuthorized($order->getTotalDue());
        $payment->setBaseAmountAuthorized($order->getBaseTotalDue());

        $order->addStatusHistoryComment(sprintf(
            'A charge permission has been processed by Amazon Payments (%s). The new status is %s.',
            $chargePermission['chargePermissionId'],
            $chargePermission['statusDetails']['state']
        ));

        $charge = $this->_getApi()->getCharge($checkoutSession->chargeId);

        /** @var Mage_Sales_Model_Order_Payment_Transaction $capture */
        $capture = $payment->setTransactionId($checkoutSession->chargeId)
            ->addTransaction(Mage_Sales_Model_Order_Payment_Transaction::TYPE_CAPTURE);

        switch ($paymentAction) {
            case Creativestyle_AmazonCheckout_Model_Payment::ACTION_AUTHORIZE_CAPTURE:
                $capture->setIsClosed(true);
                $payment->registerCaptureNotification($order->getBaseTotalDue(), true);
                break;
            default:
                $capture->setIsClosed(false);
                break;
        }

        $order->addStatusHistoryComment(sprintf(
            'A charge has been processed by Amazon Payments (%s). The new status is %s.',
            $charge['chargeId'],
            $charge['statusDetails']['state']
        ));

        $orderStatus = $this->_getConfig()->getAuthorizedOrderStatus($order->getStore());

        if (!$orderStatus) {
            $orderStatus = Mage::getModel('sales/order_status')
                ->setStore($order->getStore())
                ->loadDefaultByState(Mage_Sales_Model_Order::STATE_PROCESSING)
                ->getStatus();
        }

        $stateObject->setState(Mage_Sales_Model_Order::STATE_PROCESSING);
        $stateObject->setStatus($orderStatus);
        $stateObject->setIsNotified(false);

        return $this;
    }

    public function capture(Varien_Object $payment, $amount)
    {
        /** @var Mage_Sales_Model_Order_Payment $payment */
        $captureTransaction = $payment->lookupTransaction(null, Mage_Sales_Model_Order_Payment_Transaction::TYPE_CAPTURE);
        if ($captureTransaction && !$captureTransaction->getIsClosed()) {
            $payment->setSkipTransactionCreation(true);
            $this->_getApi()
                ->setStoreId($payment->getOrder()->getStoreId())
                ->captureCharge($captureTransaction->getTxnId(), $amount, $payment->getOrder()->getBaseCurrencyCode());
            $captureTransaction->setIsClosed(true)->save();
        }

        return $this;
    }

    public function refund(Varien_Object $payment, $amount)
    {
        /** @var Mage_Sales_Model_Order_Payment $payment */
        $captureTransaction = $payment->lookupTransaction(null, Mage_Sales_Model_Order_Payment_Transaction::TYPE_CAPTURE);
        if ($captureTransaction) {
            $this->_getApi()
                ->setStoreId($payment->getOrder()->getStoreId())
                ->createRefund($captureTransaction->getTxnId(), $amount, $payment->getOrder()->getBaseCurrencyCode());
        }

        return $this;
    }
}
